/* Example of interrupt handle LinCon8k slot.

   Author: Moki Matsushima

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   v 0.0.0  1 Sep 2004 by Moki Matsushima
     create, blah blah... */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <signal.h>

#include "msw.h"

void sig_handler(int sig)
{
	static unsigned sig_counter;
	printf("\rGot single %d for %u times ", sig, ++sig_counter);
}

int main()
{
	int fd;
	char *dev_file;
	slot_reg_t reg;
	slot_signal_t sig;

	static struct sigaction act, act_old;

	dev_file = "/dev/slot0";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* set action for signal */
	act.sa_handler = sig_handler;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, SIGALRM);
	if (sigaction(SIGALRM, &act, &act_old)) {
		close(fd);
		puts("Failure of signal action.");
		return FAILURE;
	}


	/* signal condiction */
	sig.sid = SIGALRM;
	sig.pid = getpid();
	sig.is = 1;				/* INT_CHAN_0 */
	sig.bedge = 1;			/* signal for both edges */
	if (ioctl(fd, SLOT_SET_SIG, &sig)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of signal condiction.");
		return FAILURE;
	}

	/* wait for exit */
	puts("press <enter> to exit\n");
	while (getchar() != 10);

	close(fd);
	sigaction(SIGALRM, &act_old, NULL);
	puts("\nEnd of program.");
	return SUCCESS;
}
